//==========================  Open Steamworks  ================================
//
// This file is part of the Open Steamworks project. All individuals associated
// with this project do not claim ownership of the contents
// 
// The code, comments, and all related files, projects, resources,
// redistributables included with this project are Copyright Valve Corporation.
// Additionally, Valve, the Valve logo, Half-Life, the Half-Life logo, the
// Lambda logo, Steam, the Steam logo, Team Fortress, the Team Fortress logo,
// Opposing Force, Day of Defeat, the Day of Defeat logo, Counter-Strike, the
// Counter-Strike logo, Source, the Source logo, and Counter-Strike Condition
// Zero are trademarks and or registered trademarks of Valve Corporation.
// All other trademarks are property of their respective owners.
//
//=============================================================================

#ifndef ICLIENTAUDIO_H
#define ICLIENTAUDIO_H
#ifdef _WIN32
#pragma once
#endif

#include "SteamTypes.h"
//#include "AudioCommon.h"


typedef uint32 HAudio;
typedef uint32 HFileSource;

#define CLIENTAUDIO_INTERFACE_VERSION "CLIENTAUDIO_INTERFACE_VERSION001"


abstract_class UNSAFE_INTERFACE IClientAudio
{
public:

	enum EPlayMode
	{
	};

	virtual void StartVoiceRecording() = 0;
	virtual void StopVoiceRecording() = 0;
	virtual void ResetVoiceRecording() = 0;

	// Determine the amount of captured audio data that is available in bytes.
	// This provides both the compressed and uncompressed data. Please note that the uncompressed
	// data is not the raw feed from the microphone: data may only be available if audible 
	// levels of speech are detected.
	// nUncompressedVoiceDesiredSampleRate is necessary to know the number of bytes to return in pcbUncompressed - can be set to 0 if you don't need uncompressed (the usual case)
	virtual EVoiceResult GetAvailableVoice(uint32 *pcbCompressed, uint32 *pcbUncompressed, uint32 nUncompressedVoiceDesiredSampleRate) = 0;
	// Gets the latest voice data from the microphone. Compressed data is an arbitrary format, and is meant to be handed back to 
	// DecompressVoice() for playback later as a binary blob. Uncompressed data is 16-bit, signed integer, 11025Hz PCM format.
	// Please note that the uncompressed data is not the raw feed from the microphone: data may only be available if audible 
	// levels of speech are detected, and may have passed through denoising filters, etc.
	// This function should be called as often as possible once recording has started; once per frame at least.
	// nBytesWritten is set to the number of bytes written to pDestBuffer. 
	// nUncompressedBytesWritten is set to the number of bytes written to pUncompressedDestBuffer. 
	// You must grab both compressed and uncompressed here at the same time, if you want both.
	// Matching data that is not read during this call will be thrown away.
	// GetAvailableVoice() can be used to determine how much data is actually available.
	virtual EVoiceResult GetVoice( bool bWantCompressed, void *pDestBuffer, uint32 cbDestBufferSize, uint32 *nBytesWritten, bool bWantUncompressed, void *pUncompressedDestBuffer, uint32 cbUncompressedDestBufferSize, uint32 *nUncompressBytesWritten, uint32 nUncompressedVoiceDesiredSampleRate ) = 0;
	// Gets the latest voice data. It should be called as often as possible once recording has started.
	// nBytesWritten is set to the number of bytes written to pDestBuffer. 
	virtual EVoiceResult GetCompressedVoice( void *pDestBuffer, uint32 cbDestBufferSize, uint32 *nBytesWritten ) = 0;
	// Decompresses a chunk of compressed data produced by GetVoice().
	// nBytesWritten is set to the number of bytes written to pDestBuffer unless the return value is k_EVoiceResultBufferTooSmall.
	// In that case, nBytesWritten is set to the size of the buffer required to decompress the given
	// data. The suggested buffer size for the destination buffer is 22 kilobytes.
	// The output format of the data is 16-bit signed at the requested samples per second.
	virtual EVoiceResult DecompressVoice( const void *pCompressed, uint32 cbCompressed, void *pDestBuffer, uint32 cbDestBufferSize, uint32 *nBytesWritten, uint32 nDesiredSampleRate ) = 0;
	// This returns the frequency of the voice data as it's stored internally; calling DecompressVoice() with this size will yield the best results
	virtual uint32 GetVoiceOptimalSampleRate() = 0;


	virtual bool PlayAudio( HAudio hAudio ) = 0;
	virtual void StopAudio() = 0;
	virtual void PauseAudio() = 0;
	virtual void UnPauseAudio() = 0;

	virtual void SetPlayOrder( IClientAudio::EPlayMode ePlayMode ) = 0;
	virtual void ShufflePlayList() = 0;
	virtual IClientAudio::EPlayMode GetPlayMode() = 0;

	virtual HAudio GetAudioHandle( const char *cszFilePath ) = 0;
	virtual bool GetAudioDetails( HAudio hAudio, char *, int32, char *, int32, char *, int32 ) = 0;
	virtual bool GetAudioArtSize( HAudio hAudio, uint32 *punWidth, uint32 *punHeight ) = 0;
	virtual bool GetAudioArtImage( HAudio hAudio, uint8 *pubBuffer, int32 cubBuffer ) = 0;

	virtual uint32 GetPlayListCount() = 0;
	virtual bool AddSongToPlaylist( HAudio hAudio ) = 0;
	virtual bool RemoveSongFromPlaylist( HAudio hAudio ) = 0;

	virtual bool SaveCurrentPlaylistToFile( const char *cszFilePath ) = 0;
	virtual bool LoadPlaylistFromFile( const char *cszFilePath ) = 0;

	virtual uint32 GetFileSourceCount() = 0;

	virtual HFileSource AddLocalFileSource( const char *cszFileSource ) = 0;
	virtual HFileSource AddSMBFileSource( const char *cszFileSource ) = 0;
	virtual HFileSource AddITunesFileSource( const char *cszFileSource ) = 0;
	virtual bool RemoveFileSource( HFileSource hFileSource ) = 0;

};

#endif // ICLIENTAUDIO_H
